@@ -4,10 +4,11 @@ import logging
 logger = logging.getLogger('nsq')
 handler = logging.StreamHandler()
 handler.setLevel(logging.DEBUG)
-formatter = logging.Formatter('[%(levelname)s] %(message)s')
+formatter = logging.Formatter(
+    '%(asctime)s [%(levelname)s] %(filename)s@%(lineno)d: %(message)s')
 handler.setFormatter(formatter)
 logger.addHandler(handler)
-logger.setLevel(logging.FATAL)
+logger.setLevel(logging.INFO)
 
 # Our underlying json implmentation
 try:
@@ -16,4 +17,4 @@ except ImportError:  # pragma: no cover
     import json
 
 # The current version
-__version__ = '0.1.2'
+__version__ = '0.1.3'
